% CIRCLEACCURACY
% Experiment: images with a single disk in them, band-limited, random
% sub-pixel shift, radius drawn from a bi-modal distribution. The distributions
% are averaged, as if all disks were in one single image. The improved
% granulometry should be able to distinguish the two modes, but the other
% binary ones shouldn't. This must be true for sufficiently small distance
% between modes, and sufficiently small disks.

% The function GRANULOMETRY implements the various methods for Euclidean disks,
% including the method proposed here. This function is part of DIPimage, which
% you will need to run this experiment. See http://www.qi.tnw.tudelft.nl/DIPlib/
% It was included here for reference.
%
% The functions GRANUL_RECT, GRANUL_OCTAGON, GRANUL_PERIODIC and GRANUL_PDE
% implement the other methods.

% Cris Luengo, July-August 2006

%clear all

R = [3,5];                  % The two radii
sz = [50,50];               % The image size
N = 50;                     % Number of objects of each size
w = 2.^((4:20)/4);          % Euclidean kernel sizes
rw = unique(round(w*sqrt(pi)/2)); % rectangular kernel sizes, integer-sized, equivalent diameter
ow = unique(round((w+1)/2)*2-1);  % octagonal kernel sizes, odd-sized
sw = [];                    % Jones and Soille's granulometry sizes, computed later by GRANUL_PERIODIC.

rb = zeros(length(rw),N*2); % rectangular granulometry
ob = zeros(length(ow),N*2); % octagonal granulometry
eb = zeros(length(w),N*2);  % Euclidean granulometry
ib = zeros(length(w),N*2);  % improved granulometry
sb = zeros(10,N*2);         % Jones and Soille's granulometry
pb = zeros(length(w),N*2);  % curve evoltion / PDE granulometry

for ii=1:N   % First radius
   fprintf(1,'.');
   img = sqrt((xx(sz)+rand(1))^2+(yy(sz)+rand(1))^2);
   img = gaussianedgeclip(img-R(1),0.5);
   rb(:,ii) = granul_rect(img,rw,'dark');
   ob(:,ii) = granul_octagon(img,ow,'dark');
   eb(:,ii) = granulometry(img,w,1,1e4,1,1,{});
   ib(:,ii) = granulometry(img,w,8,32,1,4,{'usecenter'});
   [sb(:,ii),sw] = granul_periodic(img,'dark');
   pb(:,ii) = granul_pde(img,w,'dark');
end
fprintf(1,'\n');
for ii=N+1:N+N   % Second radius
   fprintf(1,'.');
   img = sqrt((xx(sz)+rand(1))^2+(yy(sz)+rand(1))^2);
   img = gaussianedgeclip(img-R(2),0.5);
   rb(:,ii) = granul_rect(img,rw,'dark');
   ob(:,ii) = granul_octagon(img,ow,'dark');
   eb(:,ii) = granulometry(img,w,1,1e4,1,1,{});
   ib(:,ii) = granulometry(img,w,8,32,1,4,{'usecenter'});
   sb(:,ii) = granul_periodic(img,'dark');
   pb(:,ii) = granul_pde(img,w,'dark');
end
fprintf(1,'\n');

rc = sum(rb,2)'/(2*N);
rd = diff(rc,[],2)./diff(rw);
rw = rw*2/sqrt(pi); % rectangular kernel sizes, equivalent diameter

oc = sum(ob,2)'/(2*N);
od = diff(oc,[],2)./diff(ow);

ec = sum(eb,2)'/(2*N);
ed = diff(ec,[],2)./diff(w);

ic = sum(ib,2)'/(2*N);
id = diff(ic,[],2)./diff(w);

sc = sum(sb,2)'/(2*N);
sd = diff(sc,[],2)./diff(sw);

pc = sum(pb,2)'/(2*N);
pd = diff(pc,[],2)./diff(w);

clear ii img
save circleaccuracy

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure('position',[50,650,800,420]);

subplot(2,3,1)
plot(rw,rb)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('rectangular')

subplot(2,3,2)
plot(ow,ob)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('octagonal')

subplot(2,3,3)
plot(sw,sb)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('periodic')

subplot(2,3,4)
plot(w,eb)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('Euclidean')

subplot(2,3,5)
plot(w,ib)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('improved')

subplot(2,3,6)
plot(w,pb)
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,1])
hold on
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
title('curve evolution')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure('position',[500,650,800,420]);

subplot(2,3,1), hold on, box on
bar_ex(rw,rd,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,ceil(max(rd)*20)/20])
title('rectangular')

subplot(2,3,2), hold on, box on
bar_ex(ow,od,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,ceil(max(od)*20)/20])
title('octagonal')

subplot(2,3,3), hold on, box on
bar_ex(sw,sd,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,ceil(max(sd)*20)/20])
title('periodic')

subplot(2,3,4), hold on, box on
bar_ex(w,ed,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[floor(min(ed)*20)/20,ceil(max(ed)*20)/20])
title('Euclidean')

subplot(2,3,5), hold on, box on
bar_ex(w,id,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,ceil(max(id)*20)/20])
title('improved')

subplot(2,3,6), hold on, box on
bar_ex(w,pd,[0,0,0],[0.75,0.75,0.75])
plot(2*R([1,1]),[0,1],'k:')
plot(2*R([2,2]),[0,1],'k:')
set(gca,'xScale','log','xlim',[2,20],'xtick',w(1:4:end),'xticklabel',w(1:4:end),'ylim',[0,ceil(max(pd)*20)/20])
title('curve evolution')

set(gcf,'PaperPositionMode','Auto')
print -deps2 -r600 circleaccuracy.eps
