%GRANUL_PDE   Obtains a particle size distribution
%  distr = granul_pde(in,widths,polarity)
%  returns a granulometry using isotropic structuring elements implemented
%  through curve evolution.
%
% PARAMETERS:
%  in: input image
%  widths: sample locations; these are the widths of the rectangles used as SEs.
%  polarity: whether to use openings or closings. One of these options:
%     'dark': uses closings and measures dark particles agains a light background.
%     'light': uses openings and measures light particles against a dark background.

% Cris Luengo, July-August 2006

function distr = granul_pde(in,widths,polarity)

% Check input
if isa(in,'dip_image_array') | isempty(in) | ndims(in)~=2
   error('2D scalar input image expected.')
end
if ~isnumeric(widths) | ~isvector(widths)
   error('WIDTHS should be a numeric vector')
end
if any(diff(widths)<=0)
   error('WIDTHS must be monotonically increasing')
end
if strcmp(polarity,'dark')
   polarity = 0;
else
   polarity = 1;
end

% Compute distribution
levels = length(widths);
distr = zeros(1,levels);
maxim = max(in);
minim = min(in);
offset = mean(in);
if polarity==0 % this is a closing sieve..
   gain = 1/(maxim-offset);
else % this is an opening sieve...
   gain = 1/(minim-offset);
end
partial = in; % store dilated image to avoid repeating the same thing over and over.
curlevel = 0;
for level=1:levels
   if polarity==0
      partial = dilation_pde(partial,widths(level)-curlevel);
      tmp = -dilation_pde(-partial,widths(level));
   else
      partial = -dilation_pde(-partial,widths(level)-curlevel);
      tmp = dilation_pde(partial,widths(level));
   end
   curlevel = widths(level);
   tmp = mean(tmp);
   distr(level) = (tmp-offset)*gain;
end


% The dilation using a PDE.
%    erosion_pde(out,s) = -dilation_pde(-out,s)
function out = dilation_pde(out,s)
out = double(out); % makes DIFF possible and also allows JIT on loop
s = s/2;           % convert diameter parameter into a radial distance
t = 0.01;          % the step size; this seems more than small enough
emptycol = zeros(size(out,1),1);
emptyrow = zeros(1,size(out,2));
for ii=1:(s/t)
   Dx = diff(out,1,2);
   Dxp = [Dx,emptycol];
   Dxm = [emptycol,Dx];
   Dy = diff(out,1,1);
   Dyp = [Dy;emptyrow];
   Dym = [emptyrow;Dy];
   H = sqrt( min(0,Dxm).^2 + max(0,Dxp).^2 + min(0,Dym).^2 + max(0,Dyp).^2 );
   out = out + H.*t;
end
out = dip_image(out);
