%GRANUL_PERIODIC   Obtains a particle size distribution
%  [distr,widths] = granul_periodic(in,polarity)
%  returns a granulometry using Jones and Soille's approximation to
%  isotropic disks.
%
% PARAMETERS:
%  in: input image
%  polarity: whether to use openings or closings. One of these options:
%     'dark': uses closings and measures dark particles agains a light background.
%     'light': uses openings and measures light particles against a dark background.
%  widths: sample locations; these are the diamters of the disks used as SEs.

% Cris Luengo, July-August 2006

function [distr,widths] = granul_periodic(in,polarity)

% Check input
if isa(in,'dip_image_array') | isempty(in) | ndims(in)~=2
   error('2D scalar input image expected.')
end
if strcmp(polarity,'dark')
   polarity = 0;
else
   polarity = 1;
end

% Prepare widths array
maxsz = 4; % max length of periodic line thing, yields disks up to ~32 px diameter.
w = [];
for ii=0:maxsz
   for jj=ii:maxsz
      if norm([ii,jj])<=maxsz
         w = [w,norm([ii,jj])];
      end
   end
end
w = unique(w);
levels = length(w);

% Compute distribution
distr = zeros(1,levels);
widths = zeros(1,levels);
maxim = max(in);
minim = min(in);
offset = mean(in);
if polarity==0 % this is a closing sieve..
   gain = 1/(maxim-offset);
else % this is an opening sieve...
   gain = 1/(minim-offset);
end
for level=1:levels
   se = jonessoille_se(w(level));
   if polarity==0
      tmp = dip_closing(in,se,[0,0],'user_defined');
   else
      tmp = dip_opening(in,se,[0,0],'user_defined');
   end
   tmp = mean(tmp);
   distr(level) = (tmp-offset)*gain;
   widths(level) = 2*sqrt(sum(se)/pi);
end


% make approximation to disk SE with periodic lines
function se = jonessoille_se(sz)
if sz<1
   se = newim([3,3],'bin');
   se(1,1) = 1;
   return;
end
p = {[0,1],[1,0]}; % smallest SE!
for ii=1:sz
   for jj=ii:sz
      if gcd(ii,jj)==1 && norm([ii,jj])<=sz
         p = [p,{[ii,jj]},{[ii,-jj]}];
         if ii~=jj
            p = [p,{[jj,ii]},{[jj,-ii]}];
         end
      end
   end
end
k = zeros(size(p));
perim = 0;
for ii=1:length(p)
   l = norm(p{ii});
   k(ii) = floor(sz/l)+1;
   perim = perim + l*k(ii);
end
radius = perim/pi;

sesz = 2*ceil(radius)+1;
se = newim([sesz,sesz],'bin'); % odd-sized!
se(1,(sesz-1)/2) = 1;
for ii=1:length(p)
   se = dilationperiodic(se,k(ii),p{ii});
end


% Cheap implementation of periodic dilation
function in = dilationperiodic(in,k,p)
tmp = in;
for ii=2:k
   tmp = dip_wrap(tmp,p);
   in = in | tmp;
end
